==> mariadb-java-client-2.0.3-sources.jar: org/mariadb/jdbc/internal/util/PidFactory.java <==
/*
 *
 * MariaDB Client for Java
 *
 * Copyright (c) 2012-2014 Monty Program Ab.
 * Copyright (c) 2015-2017 MariaDB Ab.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this library; if not, write to Monty Program Ab info@montyprogram.com.
 *
 * This particular MariaDB Client for Java file is work
 * derived from a Drizzle-JDBC. Drizzle-JDBC file which is covered by subject to
 * the following copyright and notice provisions:
 *
 * Copyright (c) 2009-2011, Marcus Eriksson
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of the driver nor the names of its contributors may not be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS  AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 */

package org.mariadb.jdbc.internal.util;

import com.sun.jna.Library;
import com.sun.jna.Native;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

public class PidFactory {

    private static PidRequestInter pidRequest = null;

    /**
     * Factory method to avoid loading JNA classes every connection.
     *
     * @return factory that implement PID according to environment.
     */
    public static PidRequestInter getInstance() {
        if (pidRequest == null) {
            synchronized (PidFactory.class) {
                // check again within synchronized block to guard for race condition
                if (pidRequest == null) {
                    //initialize JNA methods
                    try {
                        Class platformClass = Class.forName("com.sun.jna.Platform");
                        @SuppressWarnings("unchecked")
                        Method isLinuxMethod = platformClass.getMethod("isLinux");
                        Boolean isLinux = (Boolean) isLinuxMethod.invoke(platformClass);
                        if (isLinux.booleanValue()) {

                            //Linux pid implementation
                            pidRequest = new PidRequestInter() {
                                @Override
                                public String getPid() {
                                    return String.valueOf(CLibrary.INSTANCE.getpid());
                                }
                            };

                        } else {

                            @SuppressWarnings("unchecked")
                            Method isWindowsMethod = platformClass.getMethod("isWindows");
                            Boolean isWindows = (Boolean) isWindowsMethod.invoke(platformClass);
                            if (isWindows.booleanValue()) {

                                //Windows pid implementation
                                pidRequest = new PidRequestInter() {
                                    @Override
                                    public String getPid() {
                                        try {
                                            Class kernel32Class = Class.forName("com.sun.jna.platform.win32.Kernel32");
                                            Field field = kernel32Class.getField("INSTANCE");
                                            Object fieldinstance = field.get(kernel32Class);
                                            Method getCurrentProcessIdMethod = fieldinstance.getClass().getMethod("GetCurrentProcessId");
                                            return String.valueOf(getCurrentProcessIdMethod.invoke(fieldinstance));
                                        } catch (Throwable cle) {
                                            //jna plateform jar's are not in classpath, no PID returned
                                        }
                                        return null;
                                    }
                                };

                            }
                        }
                    } catch (Exception cle) {
                        //jna jar's are not in classpath, no PID returned
                    }

                    //No JNA, or environment not Linux/windows -> return no PID
                    if (pidRequest == null) {
                        pidRequest = new PidRequestInter() {
                            @Override
                            public String getPid() {
                                return null;
                            }
                        };
                    }
                }
            }
        }
        return pidRequest;
    }

    public interface PidRequestInter {
        String getPid();
    }

    private interface CLibrary extends Library {
        CLibrary INSTANCE = (CLibrary) Native.loadLibrary("c", CLibrary.class);

        int getpid();
    }
}

==> mariadb-java-client-2.1.2-sources.jar: org/mariadb/jdbc/internal/util/PidFactory.java <==
/*
 *
 * MariaDB Client for Java
 *
 * Copyright (c) 2012-2014 Monty Program Ab.
 * Copyright (c) 2015-2017 MariaDB Ab.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this library; if not, write to Monty Program Ab info@montyprogram.com.
 *
 * This particular MariaDB Client for Java file is work
 * derived from a Drizzle-JDBC. Drizzle-JDBC file which is covered by subject to
 * the following copyright and notice provisions:
 *
 * Copyright (c) 2009-2011, Marcus Eriksson
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of the driver nor the names of its contributors may not be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS  AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 */

package org.mariadb.jdbc.internal.util;

import com.sun.jna.Library;
import com.sun.jna.Native;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

public class PidFactory {

    private static PidRequestInter pidRequest = null;

    /**
     * Factory method to avoid loading JNA classes every connection.
     *
     * @return factory that implement PID according to environment.
     */
    public static PidRequestInter getInstance() {
        if (pidRequest == null) {
            synchronized (PidFactory.class) {
                // check again within synchronized block to guard for race condition
                if (pidRequest == null) {
                    //initialize JNA methods
                    try {
                        Class platformClass = Class.forName("com.sun.jna.Platform");
                        @SuppressWarnings("unchecked")
                        Method isLinuxMethod = platformClass.getMethod("isLinux");
                        Boolean isLinux = (Boolean) isLinuxMethod.invoke(platformClass);
                        if (isLinux) {

                            //Linux pid implementation
                            pidRequest = () -> String.valueOf(CLibrary.INSTANCE.getpid());

                        } else {

                            @SuppressWarnings("unchecked")
                            Method isWindowsMethod = platformClass.getMethod("isWindows");
                            Boolean isWindows = (Boolean) isWindowsMethod.invoke(platformClass);
                            if (isWindows) {

                                //Windows pid implementation
                                pidRequest = () -> {
                                    try {
                                        Class kernel32Class = Class.forName("com.sun.jna.platform.win32.Kernel32");
                                        Field field = kernel32Class.getField("INSTANCE");
                                        Object fieldinstance = field.get(kernel32Class);
                                        Method getCurrentProcessIdMethod = fieldinstance.getClass().getMethod("GetCurrentProcessId");
                                        return String.valueOf(getCurrentProcessIdMethod.invoke(fieldinstance));
                                    } catch (Throwable cle) {
                                        //jna plateform jar's are not in classpath, no PID returned
                                    }
                                    return null;
                                };

                            }
                        }
                    } catch (Throwable cle) {
                        //jna jar's are not in classpath, no PID returned
                    }

                    //No JNA, or environment not Linux/windows -> return no PID
                    if (pidRequest == null) {
                        pidRequest = () -> null;
                    }
                }
            }
        }
        return pidRequest;
    }

    public interface PidRequestInter {
        String getPid();
    }

    private interface CLibrary extends Library {
        CLibrary INSTANCE = (CLibrary) Native.loadLibrary("c", CLibrary.class);

        int getpid();
    }
}
